@@ -141,8 +141,8 @@ public:
 
   // lack of device is not actually an error,
   // so it doesn't throw
-  virtual bool connect();
-  virtual bool disconnect();
+  bool connect();
+  bool disconnect();
 
   /*
    *	Sends packet of HID_REPORT_SIZE.
