@@ -76,8 +76,11 @@ def C_offline(request=None):
     return get_library(request, allow_offline=True)
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="session")
 def C(request=None):
+    import platform
+    print(f"Python version: {platform.python_version()}")
+    print(f"OS: {platform.system()} {platform.release()} {platform.version()}")
     print("Getting library with connection initialized")
     return get_library(request)
 
@@ -164,6 +167,9 @@ def library_read_declarations():
 
 
 def pytest_addoption(parser):
+    parser.addoption(
+        "--runslow", action="store_true", default=False, help="run slow tests"
+    )
     parser.addoption("--run-skipped", action="store_true",
                      help="run the tests skipped by default, e.g. adding side effects")
 
@@ -171,6 +177,10 @@ def pytest_runtest_setup(item):
     if 'skip_by_default' in item.keywords and not item.config.getoption("--run-skipped"):
         pytest.skip("need --run-skipped option to run this test")
 
+def pytest_configure(config):
+    config.addinivalue_line("markers", "slow: mark test as slow to run")
+
+
 
 def library_device_reconnect(C):
     C.NK_logout()
